@@ -30,6 +30,11 @@
   "Virtualenvwrapper for Emacs."
   :group 'python)
 
+(defcustom venv-virtualenv-command "virtualenv"
+  "The command to use to run virtualenv."
+  :type '(string)
+  :group 'virtualenvwrapper)
+
 (defcustom venv-location
   (expand-file-name (or (getenv "WORKON_HOME") "~/.virtualenvs/"))
   "The location(s) of your virtualenvs. This
@@ -47,6 +52,16 @@ to activate when one of them is found."
   :type '(repeat file)
   :group 'virtualenvwrapper)
 
+
+(defcustom venv-workon-cd
+  t
+  "If set to t, cd to the virtualenv's project root when
+activating it. Analgous to the VIRTUALENVWRAPPER_WORKON_CD
+enviornment variable in the original virtualenvwrapper"
+  :type '(boolean)
+  :group 'virtualenvwrapper)
+
+
 ;; hooks
 
 (defvar venv-premkvirtualenv-hook nil
@@ -90,6 +105,7 @@ to activate when one of them is found."
   (if (eq system-type 'windows-nt) "Scripts" "bin")
   "The name of the directory containing executables. It is system dependent.")
 
+;;;###autoload
 (defun venv-projectile-auto-workon ()
   "If a venv in the projetile root exists, activates it.
 Set your common venvs names in `venv-dirlookup-names'"
@@ -213,19 +229,32 @@ prompting the user with the string PROMPT"
 (defun venv--activate-dir (dir)
   "Given a directory corresponding to a virtualenv, activate it"
   (run-hooks 'venv-preactivate-hook)
-  (setq venv-current-dir dir)
+  (setq venv-current-dir (file-name-as-directory dir))
   ;; setup the python shell
   (setq python-shell-virtualenv-path venv-current-dir)
   ;; setup emacs exec-path
   (add-to-list 'exec-path (concat venv-current-dir venv-executables-dir))
   ;; setup the environment for subprocesses
-  (setenv "PATH" (concat venv-current-dir venv-executables-dir path-separator (getenv "PATH")))
-  ;; keep eshell path in sync
-  (setq eshell-path-env (getenv "PATH"))
+  (let ((path (concat venv-current-dir
+               venv-executables-dir
+               path-separator
+               (getenv "PATH"))))
+    (setenv "PATH" path)
+    ;; keep eshell path in sync
+    (setq eshell-path-env path))
   (setenv "VIRTUAL_ENV" venv-current-dir)
+  (if venv-workon-cd
+    (venv--switch-to-project-dir))
   (venv--set-venv-gud-pdb-command-name)
   (run-hooks 'venv-postactivate-hook))
 
+(defun venv--switch-to-project-dir ()
+  "If we find the project file, cd into that directory"
+  (let ((proj-file (expand-file-name ".project" venv-current-dir)))
+    (when (file-exists-p proj-file)
+      (cd (with-temp-buffer
+            (insert-file-contents proj-file)
+            (string-trim (buffer-string)))))))
 
 ;; potentially interactive user-exposed functions
 
@@ -254,7 +283,7 @@ prompting the user with the string PROMPT"
 This is useful e.g. when using tox."
   (interactive)
   (when (not location)
-    (setq location (read-directory-name "New virtualenv: ")))
+    (setq location (read-directory-name "New virtualenv location: " venv-location)))
   (venv-deactivate)
   (setq venv-location location)
   (when (called-interactively-p 'interactive)
@@ -306,7 +335,7 @@ identifying a virtualenv."
 (defun venv--check-executable ()
   "Verify that there is a virtualenv executable available,
 throwing an error if not"
-  (unless (executable-find "virtualenv")
+  (unless (executable-find venv-virtualenv-command)
     (error "There doesn't appear to be a virtualenv executable on
     your exec path. Ensure that you have virtualenv installed and
     that the exec-path variable is set such that virtualenv can
@@ -320,20 +349,22 @@ throwing an error if not"
   )
 
 ;;;###autoload
-(defun venv-mkvirtualenv (&rest names)
-"Create new virtualenvs NAMES. If venv-location is a single
-directory, the new virtualenvs are made there; if it is a list of
-directories, the new virtualenvs are made in the current
-default-directory."
+(defun venv-mkvirtualenv-using (interpreter &rest names)
+  "Create new virtualenvs NAMES using INTERPRETER. If venv-location
+is a single directory, the new virtualenvs are made there; if it
+is a list of directories, the new virtualenvs are made in the
+current `default-directory'."
   (interactive)
   (venv--check-executable)
-  (let ((parent-dir (if (stringp venv-location)
+  (let* ((foo (if current-prefix-arg
+                         (read-string "Python executable: ")
+                       interpreter))
+        (parent-dir (if (stringp venv-location)
                         (file-name-as-directory
                          (expand-file-name venv-location))
                       default-directory))
-        (python-exe-arg (when current-prefix-arg
-                          (concat "--python="
-                                  (read-string "Python executable: " "python"))))
+        (python-exe-arg (when foo
+                          (concat "--python=" foo)))
         (names (if names names
                  (list (read-from-minibuffer "New virtualenv: ")))))
     ;; map over all the envs we want to make
@@ -342,7 +373,7 @@ default-directory."
       (when (-contains? (venv-get-candidates) it)
         (error "A virtualenv with this name already exists!"))
       (run-hooks 'venv-premkvirtualenv-hook)
-      (shell-command (concat "virtualenv " python-exe-arg " " parent-dir it))
+      (shell-command (concat venv-virtualenv-command " " python-exe-arg " " parent-dir it))
       (when (listp venv-location)
         (add-to-list 'venv-location (concat parent-dir it)))
       (venv-with-virtualenv it
@@ -352,6 +383,15 @@ default-directory."
     ;; workon the last venv we made
     (venv-workon (car (last names)))))
 
+;;;###autoload
+(defun venv-mkvirtualenv (&rest names)
+  "Create new virtualenvs NAMES. If venv-location is a single
+directory, the new virtualenvs are made there; if it is a list of
+directories, the new virtualenvs are made in the current
+`default-directory'."
+  (interactive)
+  (apply #'venv-mkvirtualenv-using nil names))
+
 ;;;###autoload
 (defun venv-rmvirtualenv (&rest names)
   "Delete virtualenvs NAMES."
